from django_utk.db.models.fix.delete_using_manager import *
from django_utk.db.models.fix.delete_using_manager import (
    __all__ as delete_using_manager_all,
)

__all__ = [*delete_using_manager_all]
